import { IGrid, ExcelExportProperties } from '../base/interface';
import { Workbook } from '@syncfusion/ej2-excel-export';
import { ServiceLocator } from '../services/service-locator';
/**
 * @hidden
 * `ExcelExport` module is used to handle the Excel export action.
 */
export declare class ExcelExport {
    private parent;
    private isExporting;
    private theme;
    private book;
    private workSheet;
    private rows;
    private columns;
    private styles;
    private data;
    private rowLength;
    private footer;
    private expType;
    private includeHiddenColumn;
    private isCsvExport;
    private isBlob;
    private isChild;
    private blobPromise;
    private exportValueFormatter;
    private isElementIdChanged;
    private helper;
    private groupedColLength;
    private globalResolve;
    private gridPool;
    private locator;
    private l10n;
    private sheet;
    private capTemplate;
    private grpFooterTemplates;
    private footerTemplates;
    private aggIndex;
    private totalAggregates;
    /**
     * Constructor for the Grid Excel Export module.
     *
     * @param {IGrid} parent - specifies the IGrid
     * @param {ServiceLocator} locator - specifies the ServiceLocator
     * @hidden
     */
    constructor(parent?: IGrid, locator?: ServiceLocator);
    /**
     * For internal use only - Get the module name.
     *
     * @returns {string} returns the module name
     */
    private getModuleName;
    private init;
    /**
     * Export Grid to Excel file.
     *
     * @param {IGrid} grid - Defines the grid.
     * @param  {exportProperties} exportProperties - Defines the export properties of the Grid.
     * @param  {isMultipleExport} isMultipleExport - Defines is multiple Grid's are exported.
     * @param  {Workbook} workbook - Defined the Workbook if multiple Grid is exported.
     * @param  {boolean} isCsv - true if export to CSV.
     * @param {boolean} isBlob - true if isBlob is enabled.
     * @returns {Promise<any>} - Returns the map for export.
     */
    Map(grid: IGrid, exportProperties: ExcelExportProperties, isMultipleExport: boolean, workbook: Workbook, isCsv: boolean, isBlob: boolean): Promise<any>;
    private exportingSuccess;
    private processRecords;
    private processInnerRecords;
    private organiseRows;
    private processGridExport;
    private processRecordContent;
    private processGroupedRows;
    private processRecordRows;
    private processDetailTemplate;
    private setImage;
    private childGridCell;
    private processAggregates;
    private fillAggregates;
    private aggregateStyle;
    private getAggreateValue;
    private mergeOptions;
    private getColumnStyle;
    private headerRotation;
    private processHeaderContent;
    private getHeaderThemeStyle;
    private updateThemeStyle;
    private getCaptionThemeStyle;
    private getRecordThemeStyle;
    private processExcelHeader;
    private updatedCellIndex;
    private processExcelFooter;
    private getIndex;
    private parseStyles;
    destroy(): void;
}
